{
  TUTOR5.PAS

  How to handle more than just one Sprite?
  How to handle them elegant?
}

uses dos, crt, _VidKRNL, _VID111, _XLSP111;

type
  TSonic = record
    x,y        : integer; { needed for the sonic position      }
    phase      : byte;    { current animation phase ( 1 to 4 ) }
    phaseDelay : byte;    { how often shall each phase drawn, before
                            switching to the next one }
    actDelay   : byte;    { how often is the current phase shown? }
    speed      : byte;    { how fast is the sonic? }
  end;

var
  sonicGFX : array[1..4] of TSprite; { this is a array of sprites }
  titleGFX : TXLSprite;              { just another sprite, but a XL with more than 64KBytes }
  sonicPAL : TPal;                   { this is a 256 color palette }
  page1    : TPage111h;              { this is a virtual page }

  b : byte; { just needed for the loading and closing the sprites }

  { Great Idea. But don't get confused. Sonic and sonicGFX are NOT
    the same }
  Sonic        : array[1..20] of TSonic;

  currentSonic : byte; { counter for the current sonic }

  titleY   : integer;  { start position of TITLE      }
  direction: shortint; { increase or decrease titleY? }


{ Have a look at the online manual of Turbo/Borland Pascal (Keyword: STR)
  Converts any integer type to a string }
function IntToStr( HelpInt : LongInt ): string;
var
  HelpString : string;
begin
  Str( HelpInt , HelpString );
  IntToStr := HelpString;
end;


begin

  { initialise the Video111h variables. Call this procedure first }
  InitVideo111h;

  loadPalette('.\PAL\SONIC.PAL',sonicPAL);        {loads a palette}

  { Load SONIC1's to the array }
  for b := 1 to 4 do
    LoadImprocesCLB( '.\GFX\SONIC1-' + IntToStr( b ) + '.CLB', sonicGFX[ b ], sonicPAL );

  { loads TITLE1.CLB to the titleGFX sprite }
  LoadXLImprocesCLB( '.\TITLE\TITLE1.CLB', titleGFX, sonicPAL);

  { initialise the Page1 }
  InitPage111h( Page1 );

  { switch on the 640x480x64k Mode }
  SetVideoMode111h( Mode111h );

  { all outputs go now to Page1, our first virtual page }
  ActivePage111H( Page1 );

  { initialise the 20 sonics }
  for currentSonic := 1 to 20 do
    begin
      Sonic[ currentSonic ].x          := random( 320 );
      Sonic[ currentSonic ].y          := random( 200 );
      Sonic[ currentSonic ].phase      := 1 + random( 4 );
      Sonic[ currentSonic ].phaseDelay := 5;
      Sonic[ currentSonic ].actDelay   := 0;
      Sonic[ currentSonic ].speed      := 2 + random( 4 ); { set move-
                                                             speed }
    end;

  titleY    := 0; { current title Y - Position is 0 }
  direction := 1; { move title screen down }

  repeat
    { Clears the Page Page1; fills it with 0's }
    ClearPage111h(Page1);

    { draws the sprite titleGFX to screenposition 0,0 }
    PutXLSpriteWindow111h( 160, 0, titleGFX );

    { draw the 20 sonics }
    for currentSonic := 1 to 20 do
      PutSpriteWindowTrans111h(
        Sonic[ currentSonic ].x, Sonic[ currentSonic ].y,
        0, sonicGFX[ Sonic[ currentSonic ].phase ] );

    { draw another title above the sonics at 0,titleY.
      Nice effect :-) }
    PutXlSpriteWindow111h( 160, titleY, titleGFX );
    TitleY := TitleY + direction; { make the title move }
    if titleY = -200 then direction :=  1; { move down }
    if titleY =  200 then direction := -1; { move up }

    { wait for a verticale retrace of the monitor;
      use this to make smooth scrolling and flicker free animations }
    Retrace;

    { copy the virtual Page Page1 to the VisualPage; also posssible would
      be CopyP2P111h ( Page1, VisualPage ) }
    CopyP2V111h( Page1 );

    { move the 20 sonics }
    for currentSonic := 1 to 20 do
      { the "with" command makes it more easier to acces the currentSonic
      for detailed information see the online manual of Turbo/Borland
      Pascal (Keyword: WITH) }
      with Sonic[ currentSonic ] do
        begin
          if x < 639 then     { make the current Sonic move }
              inc( x, speed )
            else
              x := -30;
          { change the animations phase if necessary }
          if actDelay < phaseDelay then
              inc( actDelay )
            else
              begin
                actDelay := 1;
                if phase < 4 then
                    inc( phase )
                  else
                    phase := 1;
              end;
        end;

  until keypressed; { wait for a key pressed }

  { gives the page memory back to system }
  ClosePage111h(Page1);

  { gives the sprite memory back to system }
  for b := 1 to 4 do CloseSprite( sonicGFX[ b ] );
  CloseXLSprite( titleGFX );

  { switch back to normal text mode 80x25 }
  SetVideoMode111h( Mode3h );

  { Closes the Video111h. Should be the last instruction in your programm }
  CloseVideo111h;
end.